program kernel_edd

  implicit none

  character*80 :: input = 'zeitreihe.dat'
  character*80 :: output1 = 'drift_2.dat'
  character*80 :: output2 = 'diff_2.dat'
  real*8,parameter :: delt = 0.01d0
  integer,parameter :: nsmp = 300
  real*8 :: bw_fac = 2.0d0
  integer,parameter :: tau_max = 1
  
  integer :: ndp, tau
  real*8 :: h, h_1, mini, maxi, dx, m1, sig
  real*8 :: y, y2, x0, ker
  real*8, allocatable :: x(:)
  real*8,dimension(nsmp,tau_max) :: d1, d2, m4, d1_err, d2_err, nenner
  real*8,dimension(nsmp) :: axis
  integer :: i,j,k,ios

  print*, ' '
  print*, 'Lese Daten ...'
  
  open(10,file=input,status='old')

  ndp = -1
  ios = 0
  
  do while (ios==0)
    read(10,*,iostat=ios) x0
    ndp=ndp+1 
  end do
  
  rewind(10)
  
  print*, ndp, 'Datenpunkte'
  
  allocate(x(ndp))
  
  do i=1,ndp
    read(10,*) x(i)
  end do
  
  close(10)
  print*, 'Fertig.'
  
  print*, ''
  print*, 'Bestimme Wertebereich ...'
  
  mini = x(1)
  maxi = x(1)
  
  do i=2,ndp
    if ( x(i)<mini ) mini = x(i)
    if ( x(i)>maxi ) maxi = x(i)
  end do
  
  mini = mini - (maxi-mini)*0.1d0
  maxi = maxi + (maxi-mini)*0.1d0
  
  print*, 'Minimum:', mini
  print*, 'Maximum:', maxi
  print*, ''
  print*, 'Bestimme Koordinatenachse ...'
  
  dx = (maxi - mini)/real(nsmp-1,8)

  do i=1,nsmp
    axis(i) = mini + real(i-1,8)*dx
  end do

  print*, ''
  print*, 'Bestimme optimale Bandbreite ...'

  m1 = 0.0d0
  do i=1,ndp
    m1 = m1 + x(i)
  end do
  m1 = m1/real(ndp,8)
  print*, 'Mittelwert:',m1

  sig = 0.0d0
  do i=1,ndp
    sig = sig + (x(i) - m1)**2.0d0
  end do
  sig = sig / real(ndp-1,8)
  sig = sqrt(sig)
  print*, 'Standardabweichung:', sig

  h = bw_fac*sig*(real(ndp,8)**(-0.2d0))
  h_1 = 1.0d0/h

  print*, 'Bandbreite: ',h

  print*, ''
  print*, 'Bestimme Drift und Diffusion ...'

  d1 = 0.0d0
  d2 = 0.0d0
  nenner = 0.0d0

  do tau = 1,tau_max
    print*, 'tau', tau
    do j=1,nsmp

      x0 = axis(j)
      
    ! Berechne Nenner
      do i=1,ndp-tau
	nenner(j,tau) = nenner(j,tau) + h_1*ker(h_1*(x0-x(i)))
      end do
      
    ! Berechne Zaehler
      do i=1,ndp-tau
	y = x(i+tau) - x(i)
	y2=y*y
	d1(j,tau) = d1(j,tau) + (h_1*ker(h_1*(x0-x(i)))*y)
	d2(j,tau) = d2(j,tau) + (h_1*ker(h_1*(x0-x(i)))*y2)
	m4(j,tau) = m4(j,tau) + (h_1*ker(h_1*(x0-x(i)))*y2*y2) 
      end do
      
    end do
    
    d1(:,tau) = d1(:,tau)/nenner(:,tau)
    d2(:,tau) = d2(:,tau)/nenner(:,tau)
    m4(:,tau) = m4(:,tau)/nenner(:,tau)
    
    d1_err(:,tau) = d2(:,tau) - d1(:,tau)*d1(:,tau)
    d1_err(:,tau) = d1_err(:,tau)/nenner(:,tau)
    d1_err(:,tau) = sqrt(d1_err(:,tau))
    d1_err(:,tau) = d1_err(:,tau)*sqrt(0.518d0*h_1)
    
    d2_err(:,tau) = m4(:,tau) - d2(:,tau)*d2(:,tau)
    d2_err(:,tau) = d2_err(:,tau)/nenner(:,tau)
    d2_err(:,tau) = sqrt(d2_err(:,tau))
    d2_err(:,tau) = d2_err(:,tau)*sqrt(0.518d0*h_1)
      
    d1(:,tau) = d1(:,tau) / (real(tau,8)*delt)
    d2(:,tau) = d2(:,tau) / (real(2*tau,8)*delt)
    d1_err(:,tau) = d1_err(:,tau) / (real(tau,8)*delt)
    d2_err(:,tau) = d2_err(:,tau) / (real(2*tau,8)*delt)

  end do

  open(20,file=output1,status='replace')
  do j=1,tau_max
    do i=1,nsmp
      if (nenner(i,j)>3.0d0*h_1*ker(0.0d0)) then
	write(20,"(4(e13.6,x))") real(j,8)*delt, axis(i), d1(i,j), d1_err(i,j)
      end if
    end do
    write(20,*) ''
  end do
  close(20)
  
  open(30,file=output2,status='replace')
  do j=1,tau_max
    do i=1,nsmp
      if (nenner(i,j)>3.0d0*h_1*ker(0.0d0)) then
	write(30,"(4(e13.6,x))") real(j,8)*delt, axis(i), d2(i,j), d2_err(i,j)
      end if
    end do
    write(30,*) ''
  end do
  close(30)

end program

!========================================================================

real*8 function ker(z)

implicit none

real*8 z
real*8 :: c = 3.0d0*sqrt(5.0d0)/100.0d0

  if (z*z.le.5.0d0) then
    ker = c*(5.0d0 - z*z)
  else
    ker = 0.0d0
  end if

end function
